@@ -1,3 +1,7 @@
+# xxx need 'raw' format, like plain but no wrap for resolve mode
+# xxx or need 'granvl' format, like anvl but no wrap for resolve mode?
+# xxx need 'null' format, to do ...?
+
 package File::OM;
 
 use 5.006;
@@ -5,7 +9,7 @@ use strict;
 use warnings;
 
 our $VERSION;
-$VERSION = sprintf "%d.%02d", q$Name: Release-0-28 $ =~ /Release-(\d+)-(\d+)/;
+$VERSION = sprintf "%d.%02d", q$Name: Release-1-00 $ =~ /Release-(\d+)-(\d+)/;
 
 require Exporter;
 our @ISA = qw(Exporter);
@@ -55,6 +59,7 @@ sub om_opt_defaults { return {
 		'rec',
 	wrap		=> 72,	# at which column to wrap elements (0=nowrap)
 	wrap_indent	=> '',	# current indent for wrap, but "\t" for ANVL
+				# xxx is this even used?
 	verbose		=> 0,	# more output (default less)
 
 	# The following keys are maintained internally.
@@ -184,6 +189,71 @@ sub elem {	# OM::ANVL
 		);
 		$s .= "\n";			# close comment
 	}
+# XXX what if ref($value) eq "ARRAY" -> can be used for repeated vals?
+# XXX does undefined $name mean comment?
+# XXX document what undef for $name means
+	elsif (defined $name) {			# no element if no name
+	# XXX would it look cooler with :\t after the label??
+		# xxx this should be stacked
+		$self->{element_name} = $self->name_encode($name);
+		my $enc_val = $self->value_encode($value);	# encoded value
+		$s .= $enc_val =~ /^\s*$/ ?		# wrap() loses label of
+			"$self->{element_name}:$enc_val" :	# blank value
+			Text::Wrap::wrap(		# wrap lines; this 1st
+				$self->{element_name}	# "indent" won't break
+					. ':',		# label across lines
+				"\t",			# tab for other indents
+				$enc_val)		# main part to wrap
+		;
+		$s .= "\n";
+		# M_ELEMENT and C_ELEMENT would start here
+	}
+	$self->{outhandle} and
+		return (print { $self->{outhandle} } $s)
+	or
+		return $s;
+}
+
+# XXX need something that will spit out whole input ANVL record
+sub anvl_rec {	# OM::ANVL
+	my $self = shift;
+	my $rec = shift;
+# XXX ignore lineno for now
+	my ($name, $value, $lineno, $elemnum) = (shift, shift, shift, shift);
+	my ($s, $z) = ('', '');		# built string and catchup string
+
+	$self->{record_is_open} or	# call orec() to open record first
+		($z =  $self->orec(undef, $lineno),	# may call ostream()
+		$self->{record_is_open} = 1);
+	$self->{outhandle}	or $s .= $z;	# don't retain print status
+
+	#defined($elemnum) and
+	#	$self->{elemnum} = $elemnum
+	#or
+	#	$self->{elemnum}++;
+
+	# Parse $lineno, which is empty or has form LinenumType, where
+	# Type is either ':' (real element) or '#' (comment).
+	defined($lineno)	or $lineno = '1:';
+	my ($num, $type) =
+		$lineno =~ /^(\d*)\s*(.)/;
+
+	use Text::Wrap;		# recommends localizing next two settings
+	local $Text::Wrap::columns = $self->{wrap};
+	local $Text::Wrap::huge = 'overflow';
+
+	if ($type eq '#') {
+		$self->{element_name} = undef;	# indicates comment
+		$self->{elemnum}--;		# doesn't count as an element
+		$s .= Text::Wrap::wrap(		# wrap lines with '#' as
+			'#',			# first line "indent" and
+			'# ',			# '# ' for all other indents
+			$self->comment_encode($value)	# main part to wrap
+		);
+		$s .= "\n";			# close comment
+	}
+# XXX what if ref($value) eq "ARRAY" -> can be used for repeated vals?
+# XXX does undefined $name mean comment?
 # XXX document what undef for $name means
 	elsif (defined $name) {			# no element if no name
 	# XXX would it look cooler with :\t after the label??
@@ -271,9 +341,10 @@ sub cstream {	# OM::ANVL
 		return $s;
 }
 
+# XXX don't we need to look out for \n in names?
 sub name_encode {	# OM::ANVL
 	my ($self, $s) = (shift, shift);
-	$s		or return '';
+	defined($s)		or return '';
 	$s =~ s/^\s+//;
 	$s =~ s/\s+$//;		# trim both ends
 	$s =~ s/\s+/ /g;	# squeeze multiple \s to one space
@@ -291,7 +362,7 @@ sub name_encode {	# OM::ANVL
 
 sub value_encode {	# OM::ANVL
 	my ($self, $s, $anvl_mode) = (shift, shift, shift);
-	$s		or return '';
+	defined($s)		or return '';
 	$anvl_mode ||= 'ANVL';
 
 	my $value = $s;			# save original value
@@ -299,8 +370,9 @@ sub value_encode {	# OM::ANVL
 		$s =~ /^(\n*)/;		# always defined, often ""
 
 	# value after colon starts with either preserved newlines,
-	#	a space, or (if no value) nothing
-	my $value_start = $initial_newlines || ($value ? ' ' : '');
+	#	a space, or, if value is "" (as opposed to 0), nothing
+	my $value_start = $initial_newlines || ($value eq "" ? '' : ' ');
+	#my $value_start = $initial_newlines || ($value ? ' ' : '');
 	# xxxx is this the right place to enforce the space after ':'?
 
 	# xxx is there a linear whitespace char class??
@@ -320,7 +392,7 @@ sub value_encode {	# OM::ANVL
 
 sub comment_encode {	# OM::ANVL
 	my ($self, $s) = (shift, shift);
-	$s	or return '';
+	defined($s)	or return '';
 	$s =~ s/\n/\\n/g;			# escape \n  yyy??
 	return $s;
 }
@@ -464,7 +536,7 @@ sub cstream {	# OM::CSV
 sub name_encode {	# OM::CSV
 	# CSV names used only in header line
 	my ($self, $s) = (shift, shift);
-	$s		or return '';
+	defined($s)		or return '';
 
 	# yyy should names be put inside double quotes?
 	$s =~ s/^\s+//;
@@ -477,7 +549,7 @@ sub name_encode {	# OM::CSV
 
 sub value_encode {	# OM::CSV
 	my ($self, $s) = (shift, shift);
-	$s		or return '';
+	defined($s)		or return '';
 
 	$s =~ s/"/""/g;		# double all internal double-quotes
 	$s =~ s/^\s*/"/;
@@ -489,7 +561,7 @@ sub value_encode {	# OM::CSV
 sub comment_encode {	# OM::CSV
 	# in CSV this would be a pseudo-comment
 	my ($self, $s) = (shift, shift);
-	$s		or return '';
+	defined($s)		or return '';
 
 	$s =~ s/"/""/g;		# double all internal double-quotes
 	$s =~ s/^\s*/"/;
@@ -613,7 +685,7 @@ sub cstream {	# OM::JSON
 
 sub name_encode {	# OM::JSON
 	my ($self, $s) = (shift, shift);
-	$s	or return '';
+	defined($s)	or return '';
 	$s =~ s/(["\\])/\\$1/g;			# excape " and \
 	$s =~ s{
 		([\x00-\x1f])			# escape all control chars
@@ -672,7 +744,7 @@ sub elem {	# OM::Plain
 		);
 		$s .= "\n";			# close comment
 	}
-	elsif ($value and defined $name) {	# no element if no name
+	elsif (defined($value) and defined($name)) {	# no element if no name
 		# It is a feature of Plain not to print if value is empty.
 		$self->{element_name} = $self->name_encode($name);
 		$s .= Text::Wrap::wrap(		# wrap lines with '' as
@@ -907,7 +979,7 @@ sub cstream {	# OM::PSV
 sub name_encode {	# OM::PSV
 	# PSV names used only in header line
 	my ($self, $s) = (shift, shift);
-	$s		or return '';
+	defined($s)		or return '';
 
 	$s =~ s/^\s+//;
 	$s =~ s/\s+$//;		# trim both ends
@@ -922,7 +994,7 @@ sub name_encode {	# OM::PSV
 
 sub value_encode {	# OM::PSV
 	my ($self, $s) = (shift, shift);
-	$s		or return '';
+	defined($s)		or return '';
 
 	$s =~ s/^\s+//;
 	$s =~ s/\s+$//;		# trim both ends
@@ -938,7 +1010,7 @@ sub value_encode {	# OM::PSV
 sub comment_encode {	# OM::PSV
 	# in PSV this would be a pseudo-comment
 	my ($self, $s) = (shift, shift);
-	$s		or return '';
+	defined($s)		or return '';
 
 	$s =~ s/%/%%/g;		# to preserve literal %, double it
 				# yyy must be decoded by receiver
@@ -1073,7 +1145,7 @@ sub cstream {	# OM::Turtle
 
 sub name_encode {	# OM::Turtle
 	my ($self, $s) = (shift, shift);
-	$s	or return '';
+	defined($s)	or return '';
 	$s =~ s/(["\\])/\\$1/g;
 	return $s;
 	# \" \\
@@ -1081,14 +1153,14 @@ sub name_encode {	# OM::Turtle
 
 sub value_encode {	# OM::Turtle
 	my ($self, $s) = (shift, shift);
-	$s	or return '';
+	defined($s)	or return '';
 	$s =~ s/(["\\])/\\$1/g;
 	return $s;
 }
 
 sub comment_encode {	# OM::Turtle
 	my ($self, $s) = (shift, shift);
-	$s	or return '';
+	defined($s)	or return '';
 	$s =~ s/\n/\\n/g;			# escape \n
 	return $s;
 }
@@ -1243,7 +1315,8 @@ sub cstream {	# OM::XML
 
 sub name_encode {	# OM::XML
 	my $self = shift;
-	local $_ = shift(@_) || '';
+	local $_ = shift(@_);
+	defined($_)		or $_ = '';
 
 	s/&/&amp;/g;
 	s/'/&apos;/g;
@@ -1266,7 +1339,7 @@ sub value_encode {	# OM::XML
 
 sub comment_encode {	# OM::XML
 	my ($self, $s) = (shift, shift);
-	$s	or return '';
+	defined($s)	or return '';
 	$s =~ s/-->/--&gt;/g;
 	return $s;
 }
@@ -1284,7 +1357,7 @@ File::OM - Output Multiplexer routines
  use File::OM;              # to import routines into a Perl script
 
  $om = File::OM->new(       # make output object that creates strings in
-       $format, {           # XML, Turtle, JSON, ANVL, or Plain formats
+       $format, {           # XML, Turtle, JSON, ANVL, CSV, PSV, or Plain
    outhandle => *STDOUT,    # (opt) print string instead of returning it
    verbose => 1 });         # (opt) also output record and line numbers
 
@@ -1351,13 +1424,13 @@ such as "34:" or "6#".  The number indicates the line number (or octet
 offset, depending on the origin format) of the start of the element.  The
 letter is either ':' to indicate a real element or '#' to indicate a
 comment; if the latter, the element name has no defined meaning and the
-comment is contatined in the value.  To output an element as a comment
+comment is contained in the value.  To output an element as a comment
 without regard to line number, give $lineno as "#".
 
 B<OM> presents an object oriented interface.  The object constructor
 takes a format argument and returns C<undef> if the format is unknown.
 The returned object has methods for creating format-appropriate output
-corresponding (currently) to five output modes; for a complete
+corresponding (currently) to seven output modes; for a complete
 application of these methods, see L<File::ANVL::anvl_om>.  Nonetheless,
 an application can easily call no method but C<elem()>, as the
 necessary open (C<orec()> and C<ostream>) and close (C<crec()> and
